/******************************************************************************
* @file    adsTelEp.h
*
* Contents: This file contains Endpoint Application configuration
*           parameters.
*
* @par
* INTEL CONFIDENTIAL
* Copyright 2009 Intel Corporation All Rights Reserved.
* 
* The source code contained or described herein and all documents related to the
* source code ("Material") are owned by Intel Corporation or its suppliers or
* licensors. Title to the Material remains with Intel Corporation or its
* suppliers and licensors. The Material may contain trade secrets and proprietary
* and confidential information of Intel Corporation and its suppliers and
* licensors, and is protected by worldwide copyright and trade secret laws and
* treaty provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Intels prior express written permission.
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Intel in writing.
* 
* Include any supplier copyright notices as supplier requires Intel to use.
* Include supplier trademarks or logos as supplier requires Intel to use,
* preceded by an asterisk.
* An asterisked footnote can be added as follows: 
*   *Third Party trademarks are the property of their respective owners.
* 
* Unless otherwise agreed by Intel in writing, you may not remove or alter this
* notice or any other notice embedded in Materials by Intel or Intels suppliers
* or licensors in any way.
* 
*  version: ADS.L.1.1.0-160
******************************************************************************/
#ifndef ADS_TEL_EP_H_
#define ADS_TEL_EP_H_


/****************************************************************************/
/******************* Refapp Configuration ***********************************/
/****************************************************************************/

/*maximum number of calls supported by the system*/
#define MAX_CALLS_SUPPORTED             (MAX_EP_SUPPORTED/2)


/****************************************************************************/
/******************* Framework Configuration ********************************/
/****************************************************************************/


/*the geo code for the framework*/
#define ADS_TEL_PBX_GEO                  ADS_TEL_GEO_USA

/*The number of real time threads within the framework*/
#define ADS_TEL_PBX_NUM_RTTS             16

/*Framework threading priorities*/
#define ADS_TEL_PBX_CTRL_PRIOR           10
#define ADS_TEL_PBX_RTT_PRIOR            90




/****************************************************************************/
/******************* FUNCTION DECLARATIONS **********************************/
/****************************************************************************/

/*Init tracking arrays*/
void initPbxTrackingArrays(void);


#endif /*ADS_TEL_EP_H_*/

